// ZenLib::Int128u - 128-bit integer
// Copyright (C) 2002-2004 Jrme Martinez, Zen@MediaArea.net
//
// This software is provided 'as-is', without any express or implied
// warranty.  In no event will the authors be held liable for any damages
// arising from the use of this software.
//
// Permission is granted to anyone to use this software for any purpose,
// including commercial applications, and to alter it and redistribute it
// freely, subject to the following restrictions:
//
// 1. The origin of this software must not be misrepresented; you must not
//    claim that you wrote the original software. If you use this software
//    in a product, an acknowledgment in the product documentation would be
//    appreciated but is not required.
// 2. Altered source versions must be plainly marked as such, and must not be
//    misrepresented as being the original software.
// 3. This notice may not be removed or altered from any source distribution.
//
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// int128u
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//
// Version 0.0.1
// -------------
//
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// TODO : int128u is not ready for system different of 32 bits

#include <ZenLib/Conf.h>

class int128u
{
public:
    //Constructor/Destructor
    int128u ();
    int128u (const int128u &Value);
    int128u (const int      Value);

    //Operators
    bool     Get1      (int Offset) const;
    int8u    Get8      (int Offset) const;
    int32u   Get32     (int Offset) const;
    int128u &Set       (const int128u &Value);
    int128u &Set       (const bool     Value, int Pos=0);
    int128u &Set       (const int      Value, int Pos=0);
    int128u &Set_BE    (const char    *Value, int CharCount=16); //Big Endian
    int128u &Set_LE    (const char    *Value, int CharCount=16); //Little Endian
    int128u &Add       (const int128u &Value);
    int128u &Add       (const int      Value);
    int128u &Subtract  (const int128u &Value);
    int128u &Subtract  (const int      Value);
    int128u &ShiftLeft (const int      NumberOfBits);
    int      Compare   (const int128u &ToCompare) const;
    int      Compare   (const int     &ToCompare) const;

    void operator=  (const int128u &Value)       {Set(Value);}
    void operator=  (const int      Value)       {Set(Value);}
    void operator+  (const int128u &Value)       {Add(Value);}
    void operator+  (const int      Value)       {Add(Value);}
    void operator-  (const int128u &Value)       {Subtract(Value);}
    void operator-  (const int      Value)       {Subtract(Value);}
    bool operator<  (const int128u &Value) const {return (Compare(Value)< 0);}
    bool operator<  (const int      Value) const {return (Compare(Value)< 0);}
    bool operator>  (const int128u &Value) const {return (Compare(Value)> 0);}
    bool operator>  (const int      Value) const {return (Compare(Value)> 0);}
    bool operator<= (const int128u &Value) const {return (Compare(Value)<=0);}
    bool operator<= (const int      Value) const {return (Compare(Value)<=0);}
    bool operator>= (const int128u &Value) const {return (Compare(Value)>=0);}
    bool operator>= (const int      Value) const {return (Compare(Value)>=0);}
    bool operator== (const int128u &Value) const {return (Compare(Value)==0);}
    bool operator== (const int      Value) const {return (Compare(Value)==0);}
    bool operator!= (const int128u &Value) const {return (Compare(Value)!=0);}
    bool operator!= (const int      Value) const {return (Compare(Value)!=0);}

    //Utilities
    void ToHexString (Ztring *ToFormat) const;

private:
	int32u m_Data[4];
};